@@ -1,6 +1,6 @@
 package MooseX::Types::DBIx::Class;
 BEGIN {
-  $MooseX::Types::DBIx::Class::VERSION = '0.04';
+  $MooseX::Types::DBIx::Class::VERSION = '0.05';
 }
 # ABSTRACT: MooseX::Types for DBIx::Class objects
 
@@ -19,7 +19,7 @@ use MooseX::Types -declare => [qw(
     Schema
 )];
 
-use MooseX::Types::Moose qw(Maybe Str RegexpRef);
+use MooseX::Types::Moose qw(Maybe Str RegexpRef ArrayRef);
 use MooseX::Types::Parameterizable qw(Parameterizable);
 use Moose::Util::TypeConstraints;
 
@@ -31,25 +31,60 @@ class_type BaseRow, { class => 'DBIx::Class::Row' };
 
 class_type BaseSchema, { class => 'DBIx::Class::Schema' };
 
+sub _eq_scalar_or_array {
+    my($value, $other) = @_;
+    return 1 if ! defined $other;
+    return 1 if ! ref $other && $value eq $other;
+    return 1 if ref($other) eq 'ARRAY' && grep { $value eq $_ } @$other;
+    return 0;
+}
+
 subtype ResultSet,
-    as Parameterizable[BaseResultSet, Maybe[Str]],
+    as Parameterizable[BaseResultSet, Maybe[ArrayRef|Str]],
     where {
         my($rs, $source_name) = @_;
-        return is_BaseResultSet($rs) && (!$source_name || $rs->result_source->source_name eq $source_name);
+        return is_BaseResultSet($rs) && _eq_scalar_or_array($rs->result_source->source_name, $source_name);
+    },
+    message {
+        my($rs, $source_name) = @_;
+        $rs ||= '';
+        return sprintf(
+            '%s is not a ResultSet%s',
+            ( is_BaseResultSet($rs) ? 'ResultSet[' . $rs->result_source->source_name . ']' : qq('$rs') ),
+            ( defined $source_name ? qq([$source_name]) : '' )
+        );
     };
 
 subtype ResultSource,
-    as Parameterizable[BaseResultSource, Maybe[Str]],
+    as Parameterizable[BaseResultSource, Maybe[ArrayRef|Str]],
     where {
         my($rs, $source_name) = @_;
-        return is_BaseResultSource($rs) && (!$source_name || $rs->source_name eq $source_name);
+        return is_BaseResultSource($rs) && _eq_scalar_or_array($rs->source_name, $source_name);
+    },
+    message {
+        my($rs, $source_name) = @_;
+        $rs ||= '';
+        return sprintf(
+            '%s is not a ResultSource%s',
+            ( is_BaseResultSource($rs) ? 'ResultSource[' . $rs->source_name . ']' : qq('$rs') ),
+            ( defined $source_name ? qq([$source_name]) : '' )
+        );
     };
 
 subtype Row,
-    as Parameterizable[BaseRow, Maybe[Str]],
+    as Parameterizable[BaseRow, Maybe[ArrayRef|Str]],
     where {
         my($row, $source_name) = @_;
-        return is_BaseRow($row) && (!$source_name || $row->result_source->source_name eq $source_name);
+        return is_BaseRow($row) && _eq_scalar_or_array($row->result_source->source_name, $source_name);
+    },
+    message {
+        my($row, $source_name) = @_;
+        $row ||= '';
+        return sprintf(
+            '%s is not a Row%s',
+            ( is_BaseRow($row) ? 'Row[' . $row->result_source->source_name . ']' : qq('$row') ),
+            ( defined $source_name ? qq([$source_name]) : '' )
+        );
     };
 
 subtype Schema,
@@ -57,6 +92,11 @@ subtype Schema,
     where {
         my($schema, $pattern) = @_;
         return is_BaseSchema($schema) && (!$pattern || ref($schema) =~ m/$pattern/);
+    },
+    message {
+        my($schema, $criteria) = @_;
+        $schema ||= '';
+        return sprintf('%s is not a Schema%s', qq('$schema'), $criteria ? qq([$criteria]) : '');
     };
 1;
 
@@ -70,7 +110,7 @@ MooseX::Types::DBIx::Class - MooseX::Types for DBIx::Class objects
 
 =head1 VERSION
 
-version 0.04
+version 0.05
 
 =head1 SYNOPSIS
 
@@ -171,7 +211,7 @@ C<MooseX::Types::DBIx::Class>.  The usage should be identical.
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Oliver Charles.
+This software is copyright (c) 2011 by Brian Phillips.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
